package com.gct.diagnose.algorithm;

import com.alibaba.fastjson.JSONObject;
import com.gct.diagnose.entity.DgnsGtstandard;
import com.gct.diagnose.entity.DgnsGtstandardSetting;
import com.gct.diagnose.entity.ParamForCalDgnsGtstandard;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * <p>计算标准功图</p>
 * <p>
 * a.提取功图；
 * b.提取标准库功图；
 * c.若标准库功图为null，则直接将当前标准功图添加入标准库，若不为null下一步；
 * d.将当前功图与所有标准库内功图对比，有相似（目前仅判断了特征相似）的则库内对应标准图的计数+1，并记录最新持续时间，若均不相似进行下一步；
 * e.若标准库类别（个数）在允许范围之内，则将当前功图添加为一个新类，否则下一步；
 * f.先删除持续出现时间已经过期n天的或相对较早的标准图，之后再添加新类。
 * </p>
 * @author: NewMeanning
 * @create: 2021-02-01 16:17
 **/
@Component
public class CalDgnsGtstandard {

    /**
     * 计算标准功图
     * @param paramForCalDgnsGtstandard
     */
    public void cal(ParamForCalDgnsGtstandard paramForCalDgnsGtstandard){
        //a
        List<List<Double>> sgtList = paramForCalDgnsGtstandard.get();
        //b
        List<DgnsGtstandard> existedDataList = paramForCalDgnsGtstandard.getExistedDataList();

        if(existedDataList == null || existedDataList.size()==0){
            paramForCalDgnsGtstandard.add();
        }else{
            AtomicBoolean dataExistence = new AtomicBoolean(true);
            DgnsGtstandardSetting dgnsGtstandardSetting = paramForCalDgnsGtstandard.getDgnsGtstandardSetting();
            existedDataList.forEach(dgnsGtstandard -> {
                //d
                Double similarityNum = paramForCalDgnsGtstandard.getSimilarityNum(sgtList, dgnsGtstandard);
                if(similarityNum >= dgnsGtstandardSetting.getSimilarityNum()){
                    paramForCalDgnsGtstandard.update(dgnsGtstandard,similarityNum);
                    dataExistence.set(false);
                }

            });
            //e
            if(dataExistence.get()){
                //f
                if(existedDataList.size() >= dgnsGtstandardSetting.getLimitNum()){
                    paramForCalDgnsGtstandard.remove(existedDataList.get(0));
                }
                paramForCalDgnsGtstandard.add();
            }
        }

    }
}
