package com.gct.diagnose.algorithm;

import com.alibaba.fastjson.JSONObject;
import com.gct.diagnose.entity.DgnsGtbaseparam;
import com.gct.diagnose.entity.ParamForDiagramValidate;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author: Jyx
 * @Since: 2021/1/25 17:23
 */
public class DiagramValidate {

    private static final int gtPointsNumConditionlimit = 50;

    /**
     * 功图预识别（参数，拐点，面积等计算）
     *
     * @param gt 功图数据
     */
    public void analysisParam(double[][] gt, DgnsGtbaseparam gtbaseparam) {
        gtbaseparam.setSgtpointnum(gt.length);
        Map<String, Object> resMap = new ProblemA().findProACardL(gt, gtbaseparam.getN());
        gtbaseparam.setIslegal(Integer.parseInt(resMap.get("result").toString()));
        gtbaseparam.setRemark(resMap.get("remark").toString());
        if (gt.length > 0 && gtbaseparam.getSgtpointnum() > gtPointsNumConditionlimit) {
            for (int i = 0; i < gtbaseparam.getSgtpointnum(); i++) {
                if (gt[i][0] > gtbaseparam.getSMax()) {
                    gtbaseparam.setSMax(gt[i][0]);
                    gtbaseparam.setSMaxPointLocation(i);
                }
                if (gt[i][1] > gtbaseparam.getLMax()) {
                    gtbaseparam.setLMax(gt[i][1]);
                    gtbaseparam.setLMaxPointLocation(i);
                }
                if (gt[i][0] < gtbaseparam.getSMin()) {
                    gtbaseparam.setSMin(gt[i][0]);
                    gtbaseparam.setSMinPointLocation(i);
                }
                if (gt[i][1] < gtbaseparam.getLMin()) {
                    gtbaseparam.setLMin(gt[i][1]);
                    gtbaseparam.setLMinPointLocation(i);
                }

                //上下行对半分
                if (i < gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {
                    double h = Math.abs(gt[i][0] - gt[i + 1][0]);
                    double Si = (gt[i][1] + gt[i + 1][1]) * h / 2.0;
                    gtbaseparam.setArea(gtbaseparam.getArea() + Si);
                    gtbaseparam.setUpload(gtbaseparam.getUpload() + gt[i][1] / (gtbaseparam.getSgtpointnum() / 2 - 1));
                } else if (i >= gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {
                    double h = Math.abs(gt[i][0] - gt[i + 1][0]);
                    double Si = (gt[i][1] + gt[i + 1][1]) * h / 2.0;
                    gtbaseparam.setArea(gtbaseparam.getArea() - Si);
                    gtbaseparam.setDownload(gtbaseparam.getDownload() + gt[i][1] / (gtbaseparam.getSgtpointnum() / 2 + 1));
                }
                //---
                // 根据功图获取有效冲程
                // 算法:取下半段功图的斜率最大值跟最小值对应的点的冲程差
                if (i >= gtbaseparam.getSgtpointnum() / 2 - 1 && i < gtbaseparam.getSgtpointnum() - 1) {
                    double xl = Math.abs((gt[i][1] - gt[i + 1][1]) / (gt[i][0] - gt[i + 1][0] + 0.000001));//这里取的是个固定值，可能会有问题

                    if (i == gtbaseparam.getSgtpointnum() / 2 - 1) {
                        gtbaseparam.setSAtSlopeMinLoc(gt[i][0]);
                        gtbaseparam.setSAtSlopeMaxLoc(gt[i][0]);
                        gtbaseparam.setSlopeDownloadMin(xl);
                        gtbaseparam.setSlopeDownloadMax(xl);
                    } else {
                        if (xl < gtbaseparam.getSlopeDownloadMin()) {
                            gtbaseparam.setSlopeDownloadMin(xl);
                            gtbaseparam.setSAtSlopeMinLoc(gt[i][0]);
                        }
                        if (xl > gtbaseparam.getSlopeDownloadMax()) {
                            gtbaseparam.setSlopeDownloadMax(xl);
                            gtbaseparam.setSAtSlopeMaxLoc(gt[i][0]);
                        }
                    }
                }
                //----
            }
            gtbaseparam.setSEffect(Math.abs(gtbaseparam.getSAtSlopeMaxLoc() - gtbaseparam.getSAtSlopeMinLoc()));
            gtbaseparam.setS(gtbaseparam.getSMax() - gtbaseparam.getSMin());
            gtbaseparam.setL(gtbaseparam.getLMax() - gtbaseparam.getLMin());
            gtbaseparam.setLMid((gtbaseparam.getLMax() + gtbaseparam.getLMid()) / 2);
            try {
                gtbaseparam.setLFirstdownturnpoint(getFirstTurningPoint2(gt, gtbaseparam.getSgtpointnum()));
            } catch (Exception exception) {
                gtbaseparam.setLFirstdownturnpoint(-1);
                System.out.print("loadFistDownTurnPoint cal ERROR:" + exception.getMessage());
            }
        }
        if (gtbaseparam.getLMax() == Double.MIN_VALUE) gtbaseparam.setLMax(null);
        if (gtbaseparam.getLMin() == Double.MAX_VALUE) gtbaseparam.setLMin(null);
    }


    /**
     * @param gt         计算功图第一拐点载荷判断 供液不足间歇出液
     * @param gtPointNum 功图像素点数
     */
    public double getFirstTurningPoint2(double[][] gt, int gtPointNum) {
        int start = gtPointNum / 2;
        double minL = gt[start][1];
        double maxL = gt[start][1];
        double mins = gt[start][0];
        double maxs = gt[start][0];
        List<double[]> res = new ArrayList<>();
        res.add(gt[start]); //range [start,gtpointNum-1]
        for (int i = start + 1; i < gtPointNum; i++) {
            res.add(gt[i]);
            if (gt[i][1] < minL) minL = gt[i][1];
            if (gt[i][1] > maxL) maxL = gt[i][1];
            if (gt[i][0] > maxs) maxs = gt[i][0];
            if (gt[i][0] < mins) mins = gt[i][0];
        }
        double Ldd = maxL - minL;
        double sdd = maxs - mins;
        int countT = (int) Math.ceil(Math.sqrt(Math.pow(Ldd / 0.01, 2) + Math.pow(sdd / 0.1, 2)));
        double[][] gtBezierTemp = BezierInterpolation(res.toArray(new double[][]{}), countT, 2);
        for (int i = 0; i < gtBezierTemp.length; i++) {
            double zh = gtBezierTemp[i][1];
            double s = gtBezierTemp[i][0];
            for (int j = i + 1; j < gtBezierTemp.length; j++) {
                if (gtBezierTemp[j][0] < s) {
                    if (Math.abs(zh - gtBezierTemp[j][1]) < 0.1)
                        if (Math.abs(zh - gtBezierTemp[j][1]) < Math.abs(s - gtBezierTemp[j][0]) * 0.1)
                            return zh;
                }
            }
        }
        return 0.0;
    }

    /**
     * 贝塞尔插值fuc
     * 终将会存在硬件限制，不能真正的无限阶，但是计算功图200阶小case
     *
     * @param poss      贝塞尔曲线控制点坐标（包含起点终点）
     * @param precision 精度，需要计算的该条贝塞尔曲线上的点的数目
     * @param dimersion 该条贝塞尔曲线上的点维度，坐标轴数（二维坐标，三维坐标...）
     */
    public double[][] BezierInterpolation(double[][] poss, int precision, int dimersion) {
        //double[][] bezier_points = new double[0][];
        List<double[]> bezier_points = new ArrayList<>();
        //维度，坐标轴数（二维坐标，三维坐标...）
        if (dimersion != 2) return bezier_points.toArray(new double[][]{});
        //贝塞尔曲线控制点数（阶数），此说法不太对，但是便于理解记忆
        int number = poss.length;
        //控制点数不小于 2 ，至少为二维坐标系
        if (number < 2 || dimersion < 2)
            return bezier_points.toArray(new double[][]{});

        //double[][] result = new double[precision][];
        //计算杨辉三角
        double[] mi = new double[number];
        mi[0] = mi[1] = 1;
        for (int i = 3; i <= number; i++) {
            double[] t = new double[(i - 1)];
            System.arraycopy(mi, 0, t, 0, i - 1);
            /*for (int j = 0; j < i - 1; j++) {
                t[j] = mi[j];
            }*/
            mi[0] = mi[i - 1] = 1;
            for (int j = 0; j < i - 2; j++) {
                mi[j + 1] = t[j] + t[j + 1];
            }
        }

        //计算坐标点
        for (int i = 0; i < precision; i++) {
            double t = (double) i / precision;
            double temp[] = new double[dimersion];
            temp[0] = temp[1] = 0;
            for (int j = 0; j < dimersion; j++) {
                for (int k = 0; k < number; k++) {
                    temp[j] += Math.pow(1 - t, number - k - 1) * poss[k][j] * Math.pow(t, k) * mi[k];
                    //幂运算改为循环乘，防止溢出???
                    //var c_temp=poss[k][j]* mi[k];
                    //for (var c = 1; c <= number - k - 1; c++)
                    //    c_temp *= (1 - t);
                    //for (var c = 1; c <= k; c++)
                    //    c_temp *= t;
                    //temp += c_temp;
                }
            }
            bezier_points.add(temp);
        }
        return bezier_points.toArray(new double[][]{});
    }
}