package com.gct.diagnose.algorithm;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xusirui 2021/6/16
 */
//针对凡尔漏失的算法
// todo: 测试正确性 -- 初步测试正确，功图样例*1
public class ProblemB {

    /**
     * 寻找功图的四个角点
     *
     * @param data
     * @return [左下，左上，右上，右下]
     */
    private double[][] getApexPoints(double[][] data) {
        double[] apexLeftBottom = data[0];
        double[] apexRightTop = data[data.length / 2 - 1];
        double a = 0;
        double b = 0;
        boolean isAble2 = false;
        if (apexRightTop[0] - apexLeftBottom[0] == 0.0) {
            if (apexRightTop[1] != apexLeftBottom[1]) return null;
            a = 0;
            b = apexRightTop[1];
            isAble2 = true;
        } else {
            a = (apexRightTop[1] - apexLeftBottom[1]) / (apexRightTop[0] - apexLeftBottom[0]);
            b = apexRightTop[1] - a * apexRightTop[0];
            isAble2 = true;
        }
        if (!isAble2) return null;
        double A = a;
        double B = -1.0;
        double C = b;
        double[] apexLeftTop = null;
        double apexLeftTopD = -10;
        for (int i = 1; i < data.length / 2 - 1; i++) {
            double d = cDistanceOfPointLine(A, B, C, data[i][0], data[i][1]);
            if (d > apexLeftTopD) {
                apexLeftTopD = d;
                apexLeftTop = data[i];
            }
        }
        if (apexLeftTop == null) return null;
        double[] apexRightBottom = null;
        double apexRightBottomD = -10.0;
        for (int i = data.length / 2; i < data.length - 1; i++) {
            double d = cDistanceOfPointLine(A, B, C, data[i][0], data[i][1]);
            if (d > apexRightBottomD) {
                apexRightBottomD = d;
                apexRightBottom = data[i];
            }
        }
        if (apexRightBottom == null) return null;
        return new double[][]{apexLeftBottom, apexLeftTop, apexRightTop, apexRightBottom};
    }

    private double cDistanceOfPointLine(double a, double b, double c, double x0, double y0) {
        return Math.abs((a * x0 + b * y0 + c) / Math.sqrt(a * a + b * b));
    }

    /**
     *
     * @param data 功图数据
     * @param apexPoints 角点
     * @return [上曲，上直，下曲，下直]
     */
    public double[] cBlockArea(double[][] data, double[][] apexPoints) {
        double splitLine = (apexPoints[0][1] + apexPoints[2][1]) * (apexPoints[2][0] - apexPoints[0][0]) / 2.0;
        if (splitLine < 0) return null;
        double sLeftTopLine = (apexPoints[0][1] + apexPoints[1][1]) * (apexPoints[1][0] - apexPoints[0][0]) / 2.0;
        double sRightTopLine = (apexPoints[1][1] + apexPoints[2][1]) * (apexPoints[2][0] - apexPoints[1][0]) / 2.0;
        double sLeftBottomLine = (apexPoints[0][1] + apexPoints[3][1]) * (apexPoints[3][0] - apexPoints[0][0]) / 2.0;
        double sRightBottomLine = (apexPoints[2][1] + apexPoints[3][1]) * (apexPoints[2][0] - apexPoints[3][0]) / 2.0;
        double sTopLine = sLeftTopLine + sRightTopLine - splitLine;
        double sBottomLine = splitLine - (sLeftBottomLine + sRightBottomLine);
        double sTopCurve2 = 0;
        for (int i = 0; i <= data.length / 2 - 1 - 1; i++) {
            sTopCurve2 += Math.abs((data[i][1] + data[i + 1][1]) * (data[i + 1][0] - data[i][0]) / 2.0);
        }
        sTopCurve2 -= splitLine;
        double sBottomCurve2 = 0;
        for (int i = data.length / 2 - 1; i <= data.length - 1 - 1; i++) {
            sBottomCurve2 += Math.abs((data[i][1] + data[i + 1][1]) * (data[i + 1][0] - data[i][0]) / 2.0);
        }
        sBottomCurve2 = splitLine - sBottomCurve2;
        return new double[]{sTopCurve2, sTopLine, sBottomCurve2, sBottomLine};
    }

    /*public static void main(String[] args) {
        String sgtStr = "[[0.0, 100], [0.0, 66.08], [0.01, 88.56], [0.03, 43.04], [0.06, 44.16], [0.09, 44.96], [0.12, 45.76], [0.17, 47.68], [0.21, 48.64], [0.25, 49.12], [0.32, 48.64], [0.38, 47.84], [0.43, 47.04], [0.52, 45.6], [0.59, 44.96], [0.66, 44.8], [0.73, 44.96], [0.84, 46.08], [0.92, 46.88], [1.0, 47.36], [1.12, 48.16], [1.21, 48.16], [1.29, 48.0], [1.43, 47.04], [1.52, 46.4], [1.61, 45.92], [1.75, 45.6], [1.85, 45.6], [1.94, 45.76], [2.04, 45.92], [2.18, 46.4], [2.27, 46.56], [2.37, 46.56], [2.51, 46.4], [2.6, 46.24], [2.69, 45.92], [2.83, 45.12], [2.92, 44.8], [3.0, 44.64], [3.13, 44.32], [3.21, 44.32], [3.29, 44.48], [3.37, 44.48], [3.48, 44.32], [3.55, 44.0], [3.62, 43.68], [3.71, 43.04], [3.77, 42.56], [3.83, 42.08], [3.9, 41.28], [3.95, 41.12], [3.99, 40.96], [4.05, 40.64], [4.08, 40.64], [4.11, 40.47], [4.14, 40.64], [4.16, 77.64], [4.18, 55.47], [4.19, 40.47], [4.19, 40.32], [4.19, 40.15], [4.19, 39.68], [4.17, 39.04], [4.16, 38.56], [4.14, 37.92], [4.1, 36.0], [4.07, 35.36], [4.03, 35.04], [3.99, 35.2], [3.93, 35.84], [3.88, 36.47], [3.83, 37.11], [3.74, 38.24], [3.68, 38.08], [3.62, 37.76], [3.52, 36.79], [3.45, 36.15], [3.37, 35.68], [3.26, 35.04], [3.17, 35.2], [3.09, 35.36], [3.01, 35.84], [2.88, 36.47], [2.79, 36.64], [2.7, 36.64], [2.56, 36.47], [2.47, 36.0], [2.37, 35.68], [2.23, 35.2], [2.13, 35.04], [2.04, 35.04], [1.9, 35.36], [1.8, 35.68], [1.71, 36.0], [1.61, 36.15], [1.48, 36.47], [1.39, 36.47], [1.3, 36.32], [1.17, 36.15], [1.08, 36.15], [1.0, 36.15], [0.88, 36.32], [0.8, 36.64], [0.73, 36.96], [0.63, 37.6], [0.56, 37.92], [0.5, 38.08], [0.44, 38.4], [0.35, 38.56], [0.3, 38.56], [0.25, 38.72], [0.19, 39.04], [0.15, 39.2], [0.12, 39.52], [0.07, 40.15], [0.05, 40.47], [0.03, 40.64], [0.01, 40.96], [0.0, 41.12], [0.0, 41.12]]";
        double[][] sgt = JSONObject.parseObject(sgtStr, double[][].class);
        ProblemB problemB = new ProblemB();
        double[][] apexPoints = problemB.getApexPoints(sgt);
        for (int i=0;i<apexPoints.length;i++)
        {
            for (int j=0;j<apexPoints[i].length;j++){
                System.out.print(apexPoints[i][j]+",");
            }
            System.out.println("");
        }
        double[] res = problemB.cBlockArea(sgt, apexPoints);
        for (int i = 0; i < res.length; i++) {
            System.out.print(res[i]+",");
        }

    }*/

    public static void main(String[] args) {
        String sgt = "26.69  1.22  26.63  1.28  26.58  1.33  26.60  1.38  26.61  1.44  26.62  1.49 " +
                " 26.57  1.53  26.48  1.58  26.37  1.63  26.26  1.67  26.21  1.73  26.18  1.76 " +
                " 26.22  1.82  26.26  1.86  26.27  1.90  26.27  1.94  26.21  1.98  26.15  2.01 " +
                " 26.09  2.05  26.06  2.08  26.03  2.12  26.02  2.15  26.01  2.19  26.01  2.21 " +
                " 26.00  2.24  25.98  2.27  25.97  2.28  25.97  2.30  26.08  2.32  26.48  2.33 " +
                " 27.14  2.34  27.90  2.36  28.58  2.37  29.20  2.37  29.86  2.38  30.39  2.40 " +
                " 30.48  2.40  29.99  2.41  29.18  2.40  28.32  2.40  27.48  2.38  26.58  2.38 " +
                " 25.59  2.37  24.79  2.37  24.06  2.36  23.28  2.34  22.48  2.33  21.68  2.32 " +
                " 21.30  2.29  21.17  2.27  21.44  2.24  21.83  2.21  22.03  2.19  22.10  2.16 " +
                " 21.85  2.13  21.59  2.11  21.32  2.08  21.22  2.04  21.36  2.01  21.55  1.98 " +
                " 21.70  1.95  21.66  1.91  21.42  1.87  21.17  1.83  20.93  1.79  20.91  1.75 " +
                " 21.02  1.71  21.21  1.67  21.39  1.62  21.41  1.58  21.29  1.53  21.06  1.49 " +
                " 20.79  1.44  20.66  1.40  20.62  1.34  20.72  1.29  20.80  1.25  20.84  1.20 " +
                " 20.79  1.15  20.64  1.11  20.43  1.05  20.24  1.00  20.17  0.95  20.22  0.91 " +
                " 20.38  0.86  20.53  0.82  20.65  0.76  20.65  0.72  20.63  0.67  20.56  0.63 " +
                " 20.57  0.58  20.64  0.54  20.76  0.49  20.90  0.45  21.02  0.41  21.10  0.37 " +
                " 21.13  0.33  21.15  0.29  21.18  0.26  21.27  0.22  21.41  0.20  21.59  0.17 " +
                " 21.76  0.14  21.89  0.12  21.97  0.09  22.02  0.08  22.06  0.05  22.11  0.04 " +
                " 22.18  0.03  22.31  0.01  22.43  0.01  22.56  0.00  22.86  0.00  23.26  0.00 " +
                " 23.74  0.00  24.11  0.00  24.46  0.01  25.01  0.01  25.70  0.03  26.57  0.04 " +
                " 27.17  0.05  27.43  0.08  27.28  0.09  26.78  0.12  26.32  0.14  25.98  0.17 " +
                " 26.06  0.20  26.35  0.22  26.77  0.25  27.10  0.29  27.14  0.32  26.95  0.36 " +
                " 26.63  0.40  26.34  0.43  26.20  0.47  26.23  0.53  26.48  0.57  26.75  0.61 " +
                " 26.96  0.66  27.00  0.70  26.89  0.75  26.75  0.79  26.61  0.84  26.58  0.90 " +
                " 26.67  0.94  26.78  0.99  26.91  1.03  26.92  1.08  26.90  1.12  26.80  1.17 ";
               /* " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00  0.000  0.00 " +
                " 0.000  0.00  0.000  0.0";*/
        String[] s = sgt.split(" ");
        List<String> sgtList = Arrays.stream(s).filter(x -> !(x.length() == 0 || x.charAt(0) == ' ')).collect(Collectors.toList());
        List<String>tmp = new ArrayList<>();
        int flag = 0;
        String res = "";
       res+="[";
        for (int i = 0; i < sgtList.size(); i++) {
            tmp.add(sgtList.get(i));
            if (tmp.size()==2){
                if (flag++>0) res+=",";
                res+="["+tmp.get(1)+","+tmp.get(0)+"]";
                tmp.clear();
            }
        }
        res+="]";
        double[][] gt = JSONObject.parseObject(res, double[][].class);
        List<double[]> collect = Arrays.stream(gt).collect(Collectors.toList());
        List<double[]>ans = new ArrayList<>();
        for (int i = 0; i < collect.size(); i++) {
            if (collect.get(i)[0]==0&&collect.get(i+1)[0]==0.01){
                ans.addAll(collect.subList(i+1,collect.size()));
                ans.addAll(collect.subList(0,i+1));
                break;
            }
        }
        System.out.println(JSONObject.toJSONString(ans));
    }

}
