package com.gct.diagnose.controller;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.gct.common.core.result.Result;
import com.gct.diagnose.entity.DgnsGtbaseparam;
import com.gct.diagnose.entity.DgnsGtcvrresult;
import com.gct.diagnose.service.DgnsGtbaseparamService;
import com.gct.diagnose.service.DgnsGtcvrresultService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author xsr
 * @since 2021-04-19
 */
@RestController
@RequestMapping("/dgnsGtcvrresult")
public class DgnsGtcvrresultController {
    @Autowired
    DgnsGtcvrresultService dgnsGtcvrresultService;

    @RequestMapping(path = "/getCvrresultById/{id}", method = RequestMethod.GET)
    Result getSingleDiagramDiagnoseResultByID(@PathVariable("id") String id) {
        return Result.ok(dgnsGtcvrresultService.getById(id));
    }

    @ApiOperation(value = "更多工具-油井功图", notes = "根据时间获取诊断功图")
    @GetMapping("/getCvrresultByTime")
    public Result getCvrresultByTime(
            @ApiParam("开始时间") @RequestParam String beginTime,
            @ApiParam("结束时间") @RequestParam String endTime,
            @ApiParam("井号") @RequestParam String wellId,
            @ApiParam("当前页") @RequestParam Integer pageNum,
            @ApiParam("页面大小") @RequestParam Integer pageSize) {
       return dgnsGtcvrresultService.getCvrresultByTime(beginTime, endTime, wellId, pageNum, pageSize);
    }

    @ApiOperation(value = "更多工具-诊断算法",notes = "根据orgid、工况获取本次诊断使用的算法名称")
    @GetMapping("/getDefaultDgnsAlgorithmName")
    public Result getDgnsAlgorithm(
            @ApiParam("最小一级的组织ID") @RequestParam String orgId,
            @ApiParam("工况名") @RequestParam String gk) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        return dgnsGtcvrresultService.getDefaultDgnsAlgorithmName(orgId,gk);
    }

    @ApiOperation(value = "更多工具-诊断算法",notes = "根据orgid、工况获取本次诊断使用的算法内容")
    @GetMapping("/getDgnsAlgorithm")
    public Result getDgnsAlgorithm(
            @ApiParam("算法Id")@RequestParam Integer algorithms,
            @ApiParam("算法名称")@RequestParam String name,
            @ApiParam("组织Id")@RequestParam String orgId){
        return dgnsGtcvrresultService.getDgnsAlgorithm(algorithms,name,orgId);
    }

    @Autowired
    DgnsGtbaseparamService dgnsGtbaseparamService;

    @GetMapping("/getAllResult")
    public Result getAllResult(){
        List<DgnsGtcvrresult> collect = dgnsGtcvrresultService.list().stream().filter(x -> Integer.parseInt(x.getResult1()) > 0).collect(Collectors.toList());
        JSONArray ja = new JSONArray();
        collect.forEach(x->{
            DgnsGtbaseparam one = dgnsGtbaseparamService.getOne(
                    new QueryWrapper<DgnsGtbaseparam>().eq("well_id", x.getWellId()).eq("prod_date", x.getProdDateTime()));
            JSONObject obj = new JSONObject();
            obj.put("resultId",x.getId());
            obj.put("gtbaseparamId",one.getId());
            ja.add(obj);
        });
        return Result.ok(ja);
    }



}

