package com.gct.diagnose.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.diagnose.entity.DgnsGtgktype;
import com.gct.diagnose.service.DgnsGtcvrresultService;
import com.gct.diagnose.service.DgnsGtgktypeService;
import com.gct.diagnose.service.impl.DecisionTreeServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.gct.common.core.result.Result;

@RestController
@RequestMapping("/dgns")
@Api(value = "MainController",description = "前端获取界面数据")
@PreAuthorize("hasAuthority(dgns)")
//@CrossOrigin
public class MainController {

    @Autowired
    DgnsGtcvrresultService dgnsGtcvrresultService;

    @Autowired
    DecisionTreeServiceImpl decisionTreeService;

    @Autowired
    DgnsGtgktypeService dgnsGtgktypeService;

    @ApiOperation(value = "保存用户确认的数据",notes="根据传给前端的id保存用户确认信息")
    @PostMapping("/saveUserConfirm")
    public Result postUserConfirm(@ApiParam("诊断结果的uuid,存在问题,问题处置,措施内容,完成日期,见效日期")  @RequestBody JSONObject jsonObject){
        String id = jsonObject.getString("id");
        String problem = jsonObject.getString("problem");
        String deal = jsonObject.getString("deal");
        String dealContent = jsonObject.getString("dealContent");
        String endDate = jsonObject.getString("endDate");
        String effectDay = jsonObject.getString("effectDay");
        String username = jsonObject.getString("username");
        boolean isSave = dgnsGtcvrresultService.saveUserConfirm(id, problem, deal, dealContent, endDate, effectDay,username);
        return isSave?Result.ok("保存成功"):Result.error("保存失败，其他用户已完成当前操作");
    }
    
    @ApiOperation(value = "点击问题处置后显示给用户的内容",notes = "根据井号和问题内容传入数据")
    @GetMapping("/saveUserConfirm")
    public Result getUserConfirm(
            @ApiParam("诊断结果的uuid") String id,
            @ApiParam("诊断结果的问题类型")String problem,
            @ApiParam("井号")String wellId,
            @ApiParam("井名称")String wellName

    ){
        return Result.ok(dgnsGtcvrresultService.getUserConfirm(id,problem,wellId,wellName));
    }

    @ApiOperation(value = "获取指定诊断数据",notes = "获取井号日期查询的功图诊断数据")
    @GetMapping("/getDgnsResult/getDgnsCvrresult")
    public Result getDgnsCvrresult(@ApiParam("井号") @RequestParam("wellName")String wellId,
                                   @ApiParam("功图日期") @RequestParam("prodDate")String prodDate){
        return dgnsGtcvrresultService.getByWellNameAndProdDate(wellId,prodDate);
    }

    @ApiOperation(value = "从问题列表点击获取指定诊断数据",notes = "根据resultId获取诊断数据")
    @GetMapping("/getDgnsResult/getDgnsCvrresultById")
    public Result getDgnsCvrresultById(@ApiParam("id")@RequestParam String id){
        return dgnsGtcvrresultService.getDgnsCvrresultById(id);
    }

    @ApiOperation(value = "获取默认诊断数据",notes = "获取一个最新的默认的功图诊断数据，打开界面就能看见")
    @GetMapping("/getDgnsResult/getDefaultCvrresult")
    public Result getDefaultCvrresult(){
        return dgnsGtcvrresultService.getDefaultCvrresult();
    }

    @ApiOperation(value = "获取诊断功图的时间", notes = "返回功图诊断的时间列表提供给用户选择指定的诊断功图")
    @GetMapping("/getDgnsResult/getDgnsCvrresultTimeList")
    public Result getDgnsCvrresultTimeList(
           @ApiParam(value = "输入井号id") @RequestParam(required = false) String well_id,
           @ApiParam(value = "输入查询日期") @RequestParam(required = false) String date){
        return Result.ok(dgnsGtcvrresultService.getDgnsCvrresultTimeList(well_id,date));
    }

    @ApiOperation(value = "获取决策树参数",notes = "获取用户输入时的参考参数表")
    @GetMapping("/getDgnsResult/getDecisionTreeParam")
    public Result getDecisionTreeParam(){
        return decisionTreeService.getDecisionTreeParam();
    }

    @ApiOperation(value = "获取所有组织架构数据", notes = "获取所有组织架构数据")
    @GetMapping("/getDgnsResult/getAllOrgs")
    public Result getAllOrgs(){
        return Result.ok(decisionTreeService.getAllOrgs());
    }

    @ApiOperation(value = "获取所有算法分类", notes = "获取所有算法分类")
    @GetMapping("/getDgnsResult/getAllAlgorithms")
    public Result getAllAlgorithms(){
        JSONArray jsonArray = new JSONArray();
        for (DgnsGtgktype dgnsGtgktype : dgnsGtgktypeService.list()) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("id",dgnsGtgktype.getId());
            jsonObject.put("name",dgnsGtgktype.getGkName());
            jsonArray.add(jsonObject);
        }
        return Result.ok(jsonArray);
    }

    @ApiOperation(value = "搜索决策树",notes = "按照算法分类,应用范围,搜索关键字进行决策树模糊搜素")
    @PostMapping("/getDgnsResult/searchDecisionTree")
    public Result searchDecisionTree(@ApiParam(name = "algorithms,org,name",value = "算法分类,应用范围,搜索关键字")@RequestBody JSONObject jsonObject){
        String algorithms = jsonObject.get("algorithms").toString();
        String org = jsonObject.get("org").toString();
        String name = jsonObject.get("name").toString();
        return decisionTreeService.searchDecisionTree(algorithms,org,name);
    }

    @ApiOperation(value = "精准查询",notes = "按照算法分类,应用范围,算法名称进行精准获取")
    @PostMapping("/getDgnsResult/getDecisionTree")
    public Result getDecisionTree(@ApiParam(name = "algorithms,org,name",value = "算法分类,应用范围,搜索关键字")@RequestBody JSONObject jsonObject){
        String algorithms = jsonObject.get("algorithms").toString();
        String org = jsonObject.get("org").toString();
        String name = jsonObject.get("name").toString();
        return decisionTreeService.getDecisionTree(algorithms,org,name);
    }

    @ApiOperation(value = "保存决策树", notes = "保存用户新建的决策树")
    @PostMapping("/getDgnsResult/saveDecisionTree")
    public Result saveDecisionTreeParam(@RequestBody JSONObject jsonObject){
        return decisionTreeService.saveDecisionTreeParam(jsonObject);
    }



}
