package com.gct.diagnose.decisiontree;

import lombok.Data;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cglib.proxy.MethodInterceptor;
import org.springframework.cglib.proxy.MethodProxy;

import java.lang.reflect.Method;

/**
 * class name: ComputeInvokeMethodInterceptor
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/27 下午1:04
 */
@Slf4j
@Data
@Accessors(chain = true)
public class ComputeInvokeMethodInterceptor implements MethodInterceptor {

    private DependentDataConstructor globeData;
    private Object[] retArgs;
    private Object[] inArgs;

    @Override
    public Object intercept(Object o, Method method, Object[] args, MethodProxy methodProxy) throws Throwable {
        try {
            // deal value for inputting args
            if (args.length == 1) inArgs = (Object[]) args[0];//the args is always "Object[1]"
            if (globeData == null) log.info("Not need to set globeData");
            else if (inArgs != null && inArgs.length > 0)//need set globeData
                for (int i = 0; i < inArgs.length; i++) {
                    String key = inArgs[i].toString();
                    if (key != null && !key.isEmpty() && key.indexOf("@") == 0) {//the need figure out with char '@'
                        key = key.substring(1);
                        Object val = globeData.getGlobeDataValue(key);
                        if (val != null)
                            inArgs[i] = val;
                    }
                    if (inArgs[i] == null) log.warn("Your figured input param from name '{}' was not found", key);
                }

            Object ret = methodProxy.invokeSuper(o, args);
            if (ret == null) {
                log.info("not call back ,and don't return parameters");
                return new Object[]{};
            }
            Object[] rets = (Object[]) ret;
            //deal value for outputting args
            if (retArgs == null || retArgs.length == 0) {
                log.info("Not need to set return param to globeData");
                return rets;
            } else {
                for (int i = 0; i < retArgs.length && i < rets.length; i++)
                    globeData.putGlobeDataValue(retArgs[i].toString(), rets[i]);
                return rets;
            }
        } catch (Exception e) {
            e.printStackTrace();//非必要异常先不输出
            return new Object[]{};
        }
    }
}