package com.gct.diagnose.decisiontree;

import com.gct.diagnose.decisiontree.MataComputeTemplates.AddComputeTemplate;
import org.springframework.cglib.proxy.CallbackFilter;
import org.springframework.cglib.proxy.Enhancer;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.lang.annotation.*;
import java.lang.reflect.Method;
import java.util.ArrayList;

/**
 * class name: DecisionTreeInterpreter
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/25 下午3:41
 */
@Component
public class ComputeTemplateProxyFactory {
    private final Enhancer en = new Enhancer();

    /**
     * 适用于没有入参的 计算
     *
     * @param target class:? implement ComputeTemplate
     * @return T
     */
    public IComputeTemplate getProxy(Object target) {
        en.setSuperclass(target.getClass());
        en.setCallbackFilter(new ComputeInvokeMethodFilter());
        en.setCallback(new ComputeInvokeMethodInterceptor());
        return (IComputeTemplate) en.create();
    }

    /**
     * 适用于有入参的
     *
     * @param target
     * @param globData
     * @return
     */
    public IComputeTemplate getProxy(Object target, DependentDataConstructor globData) {

        en.setSuperclass(target.getClass());
        en.setCallbackFilter(new ComputeInvokeMethodFilter());
        en.setCallback(new ComputeInvokeMethodInterceptor().setGlobeData(globData));
        return (IComputeTemplate) en.create();
    }

    /**
     * 适用于有入参的 且有返回参的
     *
     * @param target
     * @param globData
     * @return
     */
    public IComputeTemplate getProxy(Object target, DependentDataConstructor globData, Object[] retArgs) {
        en.setSuperclass(target.getClass());
        //en.setCallbackFilter(new ComputeInvokeMethodFilter());
        en.setCallback(new ComputeInvokeMethodInterceptor().setGlobeData(globData).setRetArgs(retArgs));
        return (IComputeTemplate) en.create();
    }
    /**
     * 适用于有入参的 且有返回参的
     *
     * @param target
     * @param interceptor
     * @return
     */
    public IComputeTemplate getProxy(Object target, ComputeInvokeMethodInterceptor interceptor) {

        en.setSuperclass(target.getClass());
        en.setCallbackFilter(new ComputeInvokeMethodFilter());
        en.setCallback(interceptor);
        return (IComputeTemplate) en.create();
    }
    /**
     * interface name: IComputeTemplate
     *
     * @author lloyd
     * @version 1.0
     * @since 2021/1/23 下午2:47
     */
    public interface IComputeTemplate {

        Object[] invokeCompute(Object[] args) throws OperationNotSupportedException;

        //ArrayList<Object> invokeCompute(Object[] args) throws OperationNotSupportedException;
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Documented
    @Target(ElementType.TYPE)
    @Component
    public @interface ComputeTemplate {
        String name() default "";

        String describe() default "";

        String prefix() default "";//""gct.compute.templates.";

        /*

        TODO:生成可读模板必要属性----后面可视化的时候再做
        如:
         !(@in)
         @in +  @in
         @in || @in

         and so on..
         */
    }


    class ComputeInvokeMethodFilter implements CallbackFilter {
        @Override
        public int accept(Method method) {
            if (method.getName().equals("invokeCompute")) return 0;
            else
                return 0;//此处留作回调扩展 方法时使用
        }
    }
}


