package com.gct.diagnose.decisiontree;

import com.alibaba.fastjson.JSONObject;
import com.gct.diagnose.decisiontree.DataLoadTemplates.SampleDataLoad;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * class name: DecisionTreeGenerator
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/23 下午2:47
 */
@Data
@Accessors(chain = true)
@Scope("prototype")
@Component
public final class DecisionTreeGenerator {
    private ComputeTemplateProxyFactory exComputeFactory;

    private DependentDataConstructor globeData;

    /*private String[] initArgs;*/

    // Don't use lombok's set initArgs
    public DecisionTreeGenerator setArgs(Map<String, Object> initMap) throws NoSuchFieldException, IllegalAccessException {
        /*this.initArgs = initArgs;*/

        globeData.defaultLoadGlobeData(initMap);
        return this;
    }


    public DecisionTreeGenerator() {
    }

    @Autowired
    public DecisionTreeGenerator(ComputeTemplateProxyFactory exComputeFactory, DependentDataConstructor globeData) {
        this.exComputeFactory = exComputeFactory;
        this.globeData = globeData;
    }

    public DecisionTree generateDecisionTreeFromJson(String jsonStr) {
        return JSONObject.parseObject(jsonStr, DecisionTree.class);
    }

    public String generateJsonStrFromDecisionTree(DecisionTree tree) {
        return JSONObject.toJSONString(tree);
    }

    public Object runDecisionTree(DecisionTree tree) throws OperationNotSupportedException, IllegalStateException {
        if (tree == null) {
            globeData.putGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec(), "缺失二次诊断算法");
            return null;
        }

        if (!tree.ifNeedExecute()) {
            if (tree.describe != null && !tree.describe.isEmpty())
                globeData.putGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec(),
                        globeData.getGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec()) + tree.describe + '\n');
            return tree.getFinalOutContent();
        }
        boolean val = true;

        for (int i = 0; i < tree.exeCalTemps.length; i++) {
            String exeCalTemp = tree.exeCalTemps[i];
            ComputeTemplateProxyFactory.IComputeTemplate
                    computeProxy = exComputeFactory.getProxy(
                    TemplateConfiguration.getComputeTemplateBy(exeCalTemp, ""),
                    new ComputeInvokeMethodInterceptor()
                            .setGlobeData(globeData)
                            .setRetArgs(tree.outArgsFigure.get(i).toArray())
//                            .setInArgs(inArgs)
            );
            Object[] out = computeProxy.invokeCompute(tree.inArgsFigure.get(i).toArray());

            //最后一个节点必须为逻辑运算结果
            if (i == tree.exeCalTemps.length - 1) {
                if (out == null || out.length == 0)
                    throw new IllegalStateException("最后一个节点必须为逻辑运算结果");
                val = (boolean) out[0];
            }

        }
        if (tree.describe != null && !tree.describe.isEmpty())
            globeData.putGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec(),
                    globeData.getGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec()) + tree.describe + '\n');
        return runDecisionTree(tree.getNextNode(val));
    }
}
