package com.gct.diagnose.job;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.diagnose.entity.DgnsGtbaseparam;
import com.gct.diagnose.entity.DgnsGtstandard;
import com.gct.diagnose.service.DgnsGtbaseparamService;
import com.gct.diagnose.service.DgnsGtstandardService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * class name: DgnsGtStandardCalculateTask
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/7/2 下午2:54
 */
@Slf4j
public class DgnsGtStandardCalculateTask extends QuartzJobBean {

    @Autowired
    DgnsGtbaseparamService dgnsGtbaseparamService;

    @Autowired
    DgnsGtstandardService dgnsGtstandardService;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        String jobName = jobExecutionContext.getJobDetail().getJobDataMap().getString(JobDataKey.JobName.getDec());
        String jobGroup = jobExecutionContext.getJobDetail().getJobDataMap().getString(JobDataKey.JobGroup.getDec());

        log.info("*==>[jobName:{},jobGroup:{}]:start ", jobName, jobGroup);
        try {
            List<DgnsGtbaseparam> gtbaseparamList = dgnsGtbaseparamService.getNearlyOneHour();
            System.out.println("baseparam size = "+gtbaseparamList.size());
            if (gtbaseparamList == null || gtbaseparamList.size() == 0) return;
            gtbaseparamList = gtbaseparamList.parallelStream().
                    sorted(Comparator.comparing(DgnsGtbaseparam::getWellId)).collect(Collectors.toList());
            String curWellId = "";
            List<DgnsGtstandard> dgnsGtstandardList = null;
            List<DgnsGtstandard> addedDgnsGtstandardList = new ArrayList<>();
            int k = 0;
            for (DgnsGtbaseparam dgnsGtbaseparam : gtbaseparamList) {
                System.out.println(k++);
                System.out.println(JSONObject.toJSONString(dgnsGtbaseparam));
                if (!curWellId.equals(dgnsGtbaseparam.getWellId())) {
                    curWellId = dgnsGtbaseparam.getWellId();
                    dgnsGtstandardList = dgnsGtstandardService.list(        //todo: write mapper
                            new QueryWrapper<DgnsGtstandard>().eq("well_id", curWellId));
                    if (dgnsGtbaseparam != gtbaseparamList.get(0)) {
                        //do save data
                        addedDgnsGtstandardList = addedDgnsGtstandardList.stream().distinct().collect(Collectors.toList());
                        dgnsGtstandardService.saveOrUpdateBatch(addedDgnsGtstandardList);
                        addedDgnsGtstandardList = new ArrayList<>();
                    }
                }
                dgnsGtstandardService.executeCalculateTask(dgnsGtbaseparam, dgnsGtstandardList, addedDgnsGtstandardList);

            }
            dgnsGtstandardService.saveOrUpdateBatch(addedDgnsGtstandardList);
        } catch (Exception e) {
            log.error("[job jobName:{},jobGroup:{}]:run error ,message:{},cause:{}", e.getMessage(), e.getStackTrace());

            e.printStackTrace();
        }
        log.info("<==[jobName:{},jobGroup:{}]:completed ", jobName, jobGroup);
    }
}
