package com.gct.diagnose.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.diagnose.decisiontree.DecisionTree;
import com.gct.diagnose.decisiontree.DependentDataFieldEnum;
import com.gct.diagnose.entity.DgnsGtcvrparam;
import com.gct.diagnose.entity.Organization;
import com.gct.diagnose.feignclient.BaseDataClient;
import com.gct.diagnose.mapper.DgnsGtcvrparamMapper;
import com.gct.diagnose.service.DgnsGtcvrparamService;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author xusirui 2021/4/25
 */
@Service
public class DecisionTreeServiceImpl {

    @Autowired
    BaseDataClient baseDataClient;

    private static final Integer rootId = 0;
    private static final Boolean leftNode = true;

    @Autowired
    private DgnsGtcvrparamService dgnsGtcvrparamService;

    public JSONObject getAllOrgs() {
        String organizations = baseDataClient.getAllOrganizations();
        Result result = JSONObject.parseObject(organizations, Result.class);
        try {
            List<Organization> organizationList = result.toFiguredOutObjectList(Organization.class);
            System.out.println(JSONObject.toJSONString(organizationList));
            List<List<Organization>> sonList = new ArrayList<>();
            //获取最大的orgId，用于建图,防止下标越界
            String size = organizationList.stream().map(Organization::getOrgId).max(Comparator.comparingInt(Integer::parseInt)).get();
            if (Integer.parseInt(size) > 1000000) throw new RuntimeException("Org id 过大，请检查数据库中是否存在异常数据");
            for (int i = 0; i <= Integer.parseInt(size); i++) sonList.add(new ArrayList<>());
            //建立邻接表
            organizationList.forEach(x -> sonList.get(Integer.parseInt(x.getParentId())).add(x));
            //递归生成前端所需JSON
            String res = travelDfs(0, null, sonList);
            return JSONObject.parseObject(res);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private String travelDfs(int curId, Organization cur, List<List<Organization>> sonList) {
        String s = "";
        if (curId == 0) {
            s += "{options: [{\"value\": \"" + 0 + "\",\"label\": \"" + "默认" + "\"},";
        } else {
            s += "{\"value\": \"" + cur.getOrgId() + "\",\"label\": \"" + cur.getOrgName() + "\",\"children\": [";
        }
        List<Organization> sons = sonList.get(curId);
        for (int i = 0; i < sons.size(); i++) {
            s += travelDfs(Integer.parseInt(sons.get(i).getOrgId()), sons.get(i), sonList) + ',';
            if (i != sons.size() - 1) s += ',';
        }
        s += "]}";
        if (sons.isEmpty()) s = s.substring(0, s.length() - 16) + '}'; //子节点删除children字段
        return s;
    }

    public Result getDecisionTree(String algorithms, String org, String name) {

        Result res = null;
        try{
            HashMap<String, Object> conditionMap = new HashMap<>();
            conditionMap.put("paramid",Integer.parseInt(algorithms));
            conditionMap.put("org_id",Integer.parseInt(org));
            conditionMap.put("name",name);
            DgnsGtcvrparam dgnsGtcvrparam = dgnsGtcvrparamService.getOne(new QueryWrapper<DgnsGtcvrparam>().allEq(conditionMap));
            res = Result.ok(JSONObject.parse(dgnsGtcvrparam.getViewJson()));
        }
        catch (Exception e){
            res = Result.error("未找到"+name);
        }
        return res;
    }

    private List<List<String>>parseJSONArrayToList(JSONArray jsonArray){

        List<List<String>>list = new ArrayList<>();
        if(jsonArray!=null){
            for (int i=0;i<jsonArray.size();i++) {
                list.add(jsonArray.getJSONArray(i).toJavaList(String.class));
            }
        }
        return list;
    }
    public String parseDataToDecisionTree(String jsonData) {
        JSONObject data = JSONObject.parseObject(jsonData);
        JSONArray edges = (JSONArray) data.get("edges");
        JSONArray nodes = (JSONArray) data.get("nodes");
        Map<Integer, DecisionTree> nodeIdToDecisionTreeMap = new HashMap<>();  //节点ID到决策树节点的映射
        System.out.println(jsonData);
        for (int i = 0; i < nodes.size(); i++) {
            Integer id = Integer.parseInt(((JSONObject) nodes.get(i)).get("id").toString());
            JSONObject dtJsonObj = nodes.getJSONObject(i).getJSONObject("decisionTree");
            DecisionTree treeNode = new DecisionTree();
            System.out.println(nodes.getJSONObject(i));
            treeNode.setInArgsFigure(parseJSONArrayToList(dtJsonObj.getJSONArray("inArgsFigure")));
            treeNode.setOutArgsFigure(parseJSONArrayToList(dtJsonObj.getJSONArray("outArgsFigure")));
            treeNode.setFinalOutContent(dtJsonObj.getObject("finalOutContent",String.class));
            List<String> exeCalTemps = dtJsonObj.getJSONArray("exeCalTemps")==null?new ArrayList<>():dtJsonObj.getJSONArray("exeCalTemps").toJavaList(String.class);
            treeNode.setExeCalTemps(exeCalTemps.toArray(new String[exeCalTemps.size()]));
            treeNode.setDescribe(dtJsonObj.getObject("describe",String.class));
            nodeIdToDecisionTreeMap.put(id, treeNode);
        }
        for (int i = 0; i < edges.size(); i++) {
            Boolean lr = (Boolean) ((JSONObject) edges.get(i)).get("lr");
            Integer parentId = (Integer) ((JSONObject) edges.get(i)).get("src_node_id");
            Integer sonId = (Integer) ((JSONObject) edges.get(i)).get("dst_node_id");
            if (lr == leftNode) {
                nodeIdToDecisionTreeMap.get(parentId).setLeftNode(nodeIdToDecisionTreeMap.get(sonId));
            } else {
                nodeIdToDecisionTreeMap.get(parentId).setRightNode(nodeIdToDecisionTreeMap.get(sonId));
            }
        }
        return JSONObject.toJSONString(nodeIdToDecisionTreeMap.get(rootId));
    }

    public Result getDecisionTreeParam() {
        List<List<String>> params = new ArrayList<>();
        for (DependentDataFieldEnum param : DependentDataFieldEnum.values()) {
            if (!param.getTemp().equals("null")) {
                params.add(Arrays.asList(param.getEnDec(), param.getDec()));
            }
        }
        return Result.ok(params);
    }


    public Result searchDecisionTree(String algorithms, String org, String name) {
        HashMap<String, Object> conditionMap = new HashMap<>();
        if(algorithms.length()>0)conditionMap.put("paramid",Integer.parseInt(algorithms));
        if(algorithms.length()>0)conditionMap.put("org_id",Integer.parseInt(org));
       List<DgnsGtcvrparam> list;
       if(name.length()>0)list = dgnsGtcvrparamService.list(
                new QueryWrapper<DgnsGtcvrparam>().allEq(conditionMap).like(true, "name", name).last("limit 10"));
       else list = dgnsGtcvrparamService.list(
               new QueryWrapper<DgnsGtcvrparam>().allEq(conditionMap).last("limit 10"));
        JSONArray jsonArray = new JSONArray();
        for (DgnsGtcvrparam dgnsGtcvrparam : list) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("id",dgnsGtcvrparam.getParamid());
            jsonObject.put("value",dgnsGtcvrparam.getName());
            jsonArray.add(jsonObject);
        }
        return Result.ok(jsonArray);
    }

    public Result saveDecisionTreeParam(JSONObject jsonObject) {

        DgnsGtcvrparam dgnsGtcvrparam = new DgnsGtcvrparam();
        dgnsGtcvrparam.setViewJson(jsonObject.toJSONString());
        JSONObject expert =  jsonObject.getJSONObject("expert");

        dgnsGtcvrparam.setOrgId(Integer.parseInt(expert.get("org").toString()));
        dgnsGtcvrparam.setParamid(Integer.parseInt(expert.get("algorithms").toString()));
        dgnsGtcvrparam.setName(expert.get("name").toString());
        try{
            //决策树合法性校验
            String s = parseDataToDecisionTree(jsonObject.toString());
            dgnsGtcvrparam.setParamcontent(s);
            HashMap<String, Object> conditionMap = new HashMap<>();
            conditionMap.put("org_id",dgnsGtcvrparam.getOrgId());
            conditionMap.put("paramid",dgnsGtcvrparam.getParamid());
            conditionMap.put("name",dgnsGtcvrparam.getName());
            DgnsGtcvrparam gtcvrparam = dgnsGtcvrparamService.getOne(new QueryWrapper<DgnsGtcvrparam>().allEq(conditionMap));
            if (gtcvrparam == null){
                dgnsGtcvrparamService.save(dgnsGtcvrparam);
            }else {
                dgnsGtcvrparamService.update(dgnsGtcvrparam,new QueryWrapper<DgnsGtcvrparam>().allEq(conditionMap));
            }
        }catch (Exception e){
            e.printStackTrace();
            return Result.error("保存失败，决策树不合法");
        }
        return Result.ok("保存成功");
    }
}
