package com.gct.diagnose.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.diagnose.config.ApplicationEventPublisher;
import com.gct.diagnose.decisiontree.DecisionTree;
import com.gct.diagnose.decisiontree.DependentDataFieldEnum;
import com.gct.diagnose.entity.*;
import com.gct.diagnose.feignclient.BaseDataClient;
import com.gct.diagnose.feignclient.WnDataClient;
import com.gct.diagnose.mapper.DgnsGtcvrresultMapper;
import com.gct.diagnose.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.diagnose.service.impl.core.WarnSendEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.lang.reflect.InvocationTargetException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-04-19
 */
@Service
public class DgnsGtcvrresultServiceImpl extends ServiceImpl<DgnsGtcvrresultMapper, DgnsGtcvrresult> implements DgnsGtcvrresultService {

    @Autowired
    DgnsGtgktypeService dgnsGtgktypeService;

    @Autowired
    DgnsGtmeasureService dgnsGtmeasureService;

    @Autowired
    DgnsGtmeasureInfoService dgnsGtmeasureInfoService;

    @Autowired
    WnDataClient wnDataClient;

    @Autowired
    BaseDataClient baseDataClient;

    @Autowired
    DgnsGtcvrparamService dgnsGtcvrparamService;

    @Override
    public DgnsGtcvrresult getById(String id) {
        return baseMapper.getById(id);
    }

    //发送给前端的功图需要处理，否则显示出来不闭合
    private void dealResult(DgnsGtcvrresult dgnsGtcvrresult) {
        String processparam = dgnsGtcvrresult.getProcessparam();
        JSONObject jsonObject = JSONObject.parseObject(processparam);
        //为了让功图闭合
        JSONArray bzgtSgt = jsonObject.getJSONArray(DependentDataFieldEnum.BZGT.getEnDec());
        if (bzgtSgt!=null){
            bzgtSgt.add(JSONArray.parse(bzgtSgt.get(0).toString()));
            jsonObject.put(DependentDataFieldEnum.BZGT.getEnDec(), bzgtSgt);
        }
        JSONArray tzgtSgt = jsonObject.getJSONArray("TZGT");
        if (tzgtSgt!=null){
            tzgtSgt.add(JSONArray.parse(tzgtSgt.get(0).toString()));
            jsonObject.put("TZGT", tzgtSgt);
        }
        dgnsGtcvrresult.setProcessparam(jsonObject.toString());
        JSONArray sgt = JSONObject.parseArray(dgnsGtcvrresult.getSgt());
        if (sgt.size()>0){
            sgt.add(JSONArray.parse(sgt.get(0).toString()));
            dgnsGtcvrresult.setSgt(sgt.toString());
        }
    }

    @Override
    public Result getByWellNameAndProdDate(String wellId, String prodDate) {
        DgnsGtcvrresult dgnsGtcvrresult = baseMapper.getByWellNameAndProdDate(wellId, prodDate);
        if (dgnsGtcvrresult == null) return Result.error("当天无数据");
        dealResult(dgnsGtcvrresult);
        return Result.ok(dgnsGtcvrresult);
    }


    @Override
    public void add(DgnsGtcvrresult dgnsGtcvrresult) {
        baseMapper.insert(dgnsGtcvrresult);
    }

    @Override
    public List<DgnsGtcvrresult> getDgnsCvrresultTimeList(String well_id, String date) {
        List<DgnsGtcvrresult> createtime = baseMapper.selectList(new QueryWrapper<DgnsGtcvrresult>().select("id", "createtime", "prod_date_time"));
        return createtime;
    }

    @Override
    public Result getDefaultCvrresult() {
        //后续按偏好需求更改
        DgnsGtcvrresult dgnsGtcvrresult = baseMapper.selectOne(new QueryWrapper<DgnsGtcvrresult>().last("limit 1"));
        if (dgnsGtcvrresult == null) return Result.error("当天无数据");
        dealResult(dgnsGtcvrresult);
        return Result.ok(dgnsGtcvrresult);
    }

    @Override
    public JSONObject getUserConfirm(String id, String problem, String wellId, String wellName) {
        JSONObject jo = new JSONObject();
        jo.put("id", id);
        jo.put("wellId", wellId);
        jo.put("defalutProblem", problem);
        jo.put("wellName",wellName);
        JSONArray ja = new JSONArray();
        List<DgnsGtgktype> problemList = dgnsGtgktypeService.list();
        problemList.forEach(x -> {
            JSONArray measures = new JSONArray();
            dgnsGtmeasureService.listMaps(new QueryWrapper<DgnsGtmeasure>().select("measure").eq("id", x.getId()))
                    .stream().map(m -> m.get("measure").toString()).forEach(measures::add);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("problem", x.getGkName());
            jsonObject.put("measure", measures);
            ja.add(jsonObject);
        });
        jo.put("problemList", ja);
        return jo;
    }

    @Transactional //防止分布式并发处理问题
    @Override
    public boolean saveUserConfirm(String id, String problem, String deal, String dealContent, String endDate, String effectDay, String username) {
        DgnsGtcvrresult dgnsGtcvrresult = this.getOne(new QueryWrapper<DgnsGtcvrresult>().eq("id", UUID.fromString(id)));
        if (dgnsGtcvrresult.getProblem() != null && dgnsGtcvrresult.getProblem().length() != 0) return false;
        dgnsGtcvrresult.setProblem(problem);
        dgnsGtcvrresult.setDeal(deal);
        JSONArray ja = JSONArray.parseArray(dealContent);
        DgnsGtmeasureInfo dgnsGtmeasureInfo = new DgnsGtmeasureInfo();
        if (ja.size() == 0) {
            dgnsGtmeasureInfo.setMeasure(deal);
        } else {
            dgnsGtmeasureInfo.setMeasure(dealContent);
        }
        dgnsGtmeasureInfo.setWellId(dgnsGtcvrresult.getWellId());
        dgnsGtmeasureInfo.setEffectDay(Integer.parseInt(effectDay));
        dgnsGtmeasureInfo.setEndDate(LocalDate.parse(endDate).atTime(0, 0, 0));
        dgnsGtmeasureInfo.setProdDate(dgnsGtcvrresult.getProdDateTime());
        dgnsGtmeasureInfo.setProblem(problem);
        dgnsGtmeasureInfoService.save(dgnsGtmeasureInfo);
        dgnsGtcvrresult.setProcessparam(null); //更新时会报json转换错误，因此不必更新
        baseMapper.update(dgnsGtcvrresult, new QueryWrapper<DgnsGtcvrresult>().eq("id", UUID.fromString(dgnsGtcvrresult.getId())));

        //更新报警表确认信息
        wnDataClient.addUserConfirm(problem, dealContent, username, dgnsGtcvrresult.getId());
        return true;
    }

    @Override
    public Result getDgnsCvrresultById(String id) {
        DgnsGtcvrresult dgnsGtcvrresult = baseMapper.selectOne(new QueryWrapper<DgnsGtcvrresult>().eq("id", id));
        if (dgnsGtcvrresult == null) return Result.error("当天无数据");
        dealResult(dgnsGtcvrresult);
        return Result.ok(dgnsGtcvrresult);
    }

    @Override
    public Result getCvrresultByTime(String beginTime, String endTime, String wellId, Integer pageNum, Integer pageSize) {
        Page<DgnsGtcvrresult> page = baseMapper.selectPage(new Page<>(pageNum, pageSize), new QueryWrapper<DgnsGtcvrresult>()
                .eq("well_id", wellId)
                .between("prod_date_time", LocalDateTime.parse(beginTime.replace("+"," "), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))
                        , LocalDateTime.parse(endTime.replace("+"," "), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")))
        );
        return page.getSize() == 0 ? Result.error("无数据") : Result.ok(page);
    }

    @Override
    public Result getDefaultDgnsAlgorithmName(String orgId, String gk) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        //根据工况找到算法的id
        DgnsGtgktype gtgktype = dgnsGtgktypeService.getOne(new QueryWrapper<DgnsGtgktype>().eq("gk_name", gk));
        Integer id = gtgktype.getId();
        DgnsGtcvrparam gtcvrparam = null;
        do {
            gtcvrparam = dgnsGtcvrparamService.getOne(new QueryWrapper<DgnsGtcvrparam>()
                    .eq("paramid", id)
                    .eq("org_id", Integer.parseInt(orgId))
            );
            if (gtcvrparam == null) {
                if (orgId.equals("0")) {
                    //说明算法已经被删除了
                    return Result.error("算法不存在,可能已经被删除");
                }
                Organization org = (Organization) JSONObject.parseObject(baseDataClient.getOrgById(orgId), Result.class).toFiguredOutObject(Organization.class);
                orgId = org.getParentId();
            }
        } while (gtcvrparam == null);
        JSONObject jo = new JSONObject();
        jo.put("orgId", orgId);
        jo.put("name", gtcvrparam.getName());
        jo.put("category", id);
        return Result.ok(jo);
    }

    @Override
    public Result getDgnsAlgorithm(Integer algorithms, String name, String orgId) {
        DgnsGtcvrparam gtcvrparam = dgnsGtcvrparamService.getOne(
                new QueryWrapper<DgnsGtcvrparam>().eq("paramid", algorithms)
                        .eq("name", name)
                        .eq("org_id", Integer.parseInt(orgId))
        );
        DecisionTree tree = JSONObject.parseObject(gtcvrparam.getParamcontent(), DecisionTree.class);
        JSONObject jo = new JSONObject();
        List<String> resList = new ArrayList<>();
        resList.add(tree.getDescribe());
        deepFirstSearch(tree, resList);
        jo.put("context", resList);
        return Result.ok(jo);
    }

    @Async
    @Override
    public void asyncAddCvrresult(DgnsGtcvrresult dgnsGtcvrresult) {
        DgnsGtcvrresult one = baseMapper.selectOne(new QueryWrapper<DgnsGtcvrresult>().
                eq("well_id", dgnsGtcvrresult.getWellId()).
                eq("prod_date_time", dgnsGtcvrresult.getProdDateTime()));
        if (one == null) baseMapper.insert(dgnsGtcvrresult);
        else {
            dgnsGtcvrresult.setId(one.getId());
            baseMapper.updateById(dgnsGtcvrresult);
        }
        ApplicationEventPublisher.publishEvent(new WarnSendEvent(dgnsGtcvrresult));
    }

    private void deepFirstSearch(DecisionTree treeNode, List<String> resList) {
        if (!treeNode.hasChild()) return;
        if (treeNode.getLeftNode().hasChild()) {
            resList.add(treeNode.getRightNode().getDescribe() + "," + treeNode.getLeftNode().getDescribe());
            deepFirstSearch(treeNode.getLeftNode(), resList);
        } else {
            resList.add(treeNode.getLeftNode().getDescribe() + "," + treeNode.getRightNode().getDescribe());
            deepFirstSearch(treeNode.getRightNode(), resList);
        }
    }


}
