package com.gct.diagnose.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.diagnose.entity.DgnsGtmeasureInfo;
import com.gct.diagnose.entity.Organization;
import com.gct.diagnose.feignclient.BaseDataClient;
import com.gct.diagnose.mapper.DgnsGtmeasureInfoMapper;
import com.gct.diagnose.service.DgnsGtmeasureInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-05-12
 */
@Service
public class DgnsGtmeasureInfoServiceImpl extends ServiceImpl<DgnsGtmeasureInfoMapper, DgnsGtmeasureInfo> implements DgnsGtmeasureInfoService {

    @Autowired
    BaseDataClient baseDataClient;

    @Override
    public JSONObject getAllMeasureInfo(String orgId, Integer level, String wellId, String beginDate, String endDate,
                                        String category, String status, String schedule, Integer curPage, Integer pageSize) {
        List<Organization> orgList = null;
        //获取组织结构图
        try {
            orgList = JSONObject.parseObject(baseDataClient.getAllOrganizations(), Result.class).toFiguredOutObjectList(Organization.class);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Map<String, Organization> idOrgMap = new HashMap<>();
        orgList.forEach(x -> idOrgMap.put(x.getOrgId(), x));
        List<Organization> existList = new ArrayList<>(); //符合条件的站点
        Map<String, Organization> wellIdToOrgMap = new HashMap<>();
        if (wellId == null) {
            idOrgMap.forEach((k, v) -> {
                if (v.getOrgLevel() == 3) { //找出所有符合筛选条件的站点
                    int cnt = 3 - level;
                    String faId = k;
                    while (cnt-- > 0) {
                        faId = idOrgMap.get(faId).getParentId();
                    }
                    if (faId.equals(orgId)) {
                        existList.add(v);
                    }
                }
            });
            existList.forEach(x -> {
                Object data = JSONObject.parseObject(baseDataClient.getjh(x.getOrgId()), Result.class).getData();
                List<Map> maps = JSONArray.parseArray(data.toString(), Map.class);
                maps.forEach(m -> {
                    wellIdToOrgMap.put(m.get("id").toString(), x);
                });
            });
        } else { //找的是井号对应数据,存到map，后续处理逻辑相同
            wellIdToOrgMap.put(wellId, idOrgMap.get(orgId));
        }

        IPage<DgnsGtmeasureInfo> page = new Page<>(curPage, pageSize);
        QueryWrapper<DgnsGtmeasureInfo> wrapper = new QueryWrapper<DgnsGtmeasureInfo>()
                .between("create_time",
                        LocalDate.parse(beginDate).atTime(0, 0, 0),
                        LocalDate.parse(endDate).plusDays(1).atTime(0, 0, 0));
        if (category != null) wrapper = wrapper.eq("measure", category);
        if (status != null) wrapper = wrapper.eq("status", status);
        if (schedule != null) wrapper = wrapper.eq("schedule", schedule);
        wrapper = wrapper.in("well_id", wellIdToOrgMap.keySet());
        IPage<DgnsGtmeasureInfo> iPage = baseMapper.selectPage(page, wrapper);
        JSONObject jo = new JSONObject();
        jo.put("total", iPage.getTotal());
        JSONArray data = new JSONArray();
        AtomicInteger i = new AtomicInteger();
        iPage.getRecords().forEach(x -> {
            JSONObject rowObj = new JSONObject();
            rowObj.put("id", i.getAndIncrement());
            rowObj.put("org", wellIdToOrgMap.get(x.getWellId()).getOrgName());
            rowObj.put("wellId", x.getWellId());
            rowObj.put("problem", x.getProblem());
            rowObj.put("measure", x.getMeasure());
            rowObj.put("endDate", x.getEndDate());
            rowObj.put("status", x.getStatus());
            rowObj.put("schedule", x.getSchedule());
            rowObj.put("reason", x.getReason());
            rowObj.put("evaluate", x.getEvaluate());
            rowObj.put("arranger", x.getArranger());
            rowObj.put("responsible", x.getResponsible());
            data.add(rowObj);
        });
        jo.put("data", data);
        return jo;
    }

    @Override
    public JSONArray getMeasureInfoCount(String beginDate, String endDate, String orgId) {
        JSONArray data = new JSONArray();
        //复用接口，去除无用数据
        JSONArray orgArray = JSONObject.parseArray(
                JSONObject.parseObject(baseDataClient.getOrganization(orgId), Result.class).getData().toString());
        List<Object> orgList;
        orgList = orgArray.subList(1, orgArray.size());
        orgList.forEach(x -> {
            JSONObject jo = (JSONObject) x;
            String id = jo.getString("org_id");
            JSONArray wellIdArray = JSONObject.parseArray(
                    JSONObject.parseObject(baseDataClient.getjh(id), Result.class).getData().toString());
            Set<String> wellIdSet = wellIdArray.stream().map(y -> ((JSONObject) y).getString("id")).collect(Collectors.toSet());
            Integer notBegin = 0;
            Integer notEnd = 0;
            Integer end = 0;
            if (wellIdSet.size() != 0) { //为 0 无需查询，如果查询会出sql异常
                notBegin = baseMapper.selectCount(
                        new QueryWrapper<DgnsGtmeasureInfo>()
                                .eq("status", "已发布")
                                .eq("schedule", "未落实")
                                .in("well_id", wellIdSet).between("create_time", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1))
                );
                notEnd = baseMapper.selectCount(new QueryWrapper<DgnsGtmeasureInfo>()
                        .eq("status", "已发布")
                        .eq("schedule", "实施中")
                        .in("well_id", wellIdSet).between("create_time", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)));
                end = baseMapper.selectCount(new QueryWrapper<DgnsGtmeasureInfo>()
                        .eq("status", "已发布")
                        .eq("schedule", "已完成")
                        .in("well_id", wellIdSet).between("create_time", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)));
            }
            jo.put("notBegin", notBegin);
            jo.put("notEnd", notEnd);
            jo.put("end", end);
            jo.put("plan", notBegin + notEnd + end);
            data.add(jo);
        });
        return data;
    }

    @Override
    public Integer updateMeasureInfo(DgnsGtmeasureInfo dgnsGtmeasureInfo) {
        return baseMapper.update(dgnsGtmeasureInfo, new QueryWrapper<DgnsGtmeasureInfo>()
                .eq("well_id", dgnsGtmeasureInfo.getWellId()).eq("prod_date", dgnsGtmeasureInfo.getProdDate()));
    }

    @Override
    public JSONArray getAllMeasureInfoAnalysis(String beginDate, String endDate, String data) {
        JSONArray wellDataCount = JSONObject.parseArray(data);
        for (int i = 0; i < wellDataCount.size(); i++) {
            JSONObject jo = wellDataCount.getJSONObject(i);
            String orgId = jo.getString("Id");
            JSONArray wellIdArray = JSONObject.parseArray(
                    JSONObject.parseObject(baseDataClient.getjh(orgId), Result.class).getData().toString());
            Set<String> wellIdSet = wellIdArray.stream().map(x -> ((JSONObject) x).getString("id")).collect(Collectors.toSet());
            Integer end = 0;
            Integer plan = 0;
            Integer inTime = 0;
            if (wellIdSet.size() != 0) { //为 0 无需查询，如果查询会出sql异常
                end = baseMapper.selectCount(
                        new QueryWrapper<DgnsGtmeasureInfo>()
                                //.eq("status", "已发布")
                                .eq("schedule", "已完成")  //除非脏数据 已完成必定已发布
                                .in("well_id", wellIdSet)
                                .between("create_time", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1))
                );
                plan = baseMapper.selectCount(
                        new QueryWrapper<DgnsGtmeasureInfo>()
                                .eq("status", "已发布")
                                .in("well_id", wellIdSet)
                                .between("create_time", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1))
                );
                inTime = baseMapper.selectCount(new QueryWrapper<DgnsGtmeasureInfo>()
                        .eq("schedule", "已完成")
                        .in("well_id", wellIdSet)
                        .apply("date_part('day',finish_time::timestamp-end_date::timestamp)<1")
                );
                jo.put("APJH", plan);
                jo.put("APLS", end);
                jo.put("APLSL", plan == 0 ? 0 : end / plan * 100);

                jo.put("SSLS", end);
                jo.put("SSJS", inTime);
                jo.put("SSJSL", end == 0 ? 0 : inTime * 1.0 / end * 100);
            }
        }



        return wellDataCount;
    }


}
