package com.gct.diagnose.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.util.SGTUtil;
import com.gct.diagnose.algorithm.CalDgnsGtstandard;
import com.gct.diagnose.entity.*;
import com.gct.diagnose.mapper.DgnsGtstandardMapper;
import com.gct.diagnose.service.DgnsGtmloutcodeService;
import com.gct.diagnose.service.DgnsGtstandardService;
import com.gct.diagnose.service.DgnsGtstandardSettingService;
import com.gct.diagnose.thriftclient.GTComputerViewRecognitionClient;
import com.gct.diagnose.thriftclient.ThriftClientResponseData;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.beans.Transient;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * <p>
 *  标准功图服务实现类
 * </p>
 *
 * @author NewMeanning
 * @since 2021-02-03
 */
@Service
@Slf4j
public class DgnsGtstandardServiceImpl extends ServiceImpl<DgnsGtstandardMapper, DgnsGtstandard> implements DgnsGtstandardService {

    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private CalDgnsGtstandard calDgnsGtstandard;

    @Autowired
    private DgnsGtmloutcodeService dgnsGtmloutcodeService;

    @Autowired
    private DgnsGtstandardSettingService dgnsGtstandardSettingService;

    @Autowired
    private GTComputerViewRecognitionClient gtComputerViewRecognitionClient;

    @Override
    public void executeCalculateTask(DgnsGtbaseparam dgnsGtbaseparam, List<DgnsGtstandard> dgnsGtstandardList, List<DgnsGtstandard> addedDgnsGtstandardList) {
        log.info("{}--实时计算标准功图",Thread.currentThread().getName());
        //获取编码相关信息
        //DgnsGtmloutcode byCode = dgnsGtmloutcodeService.getByCode(zdCode);
        DgnsGtstandardSetting dgnsGtstandardSetting = dgnsGtstandardSettingService.getSetting();
        List<List<Double>> sgtList = SGTUtil.encodeToUseList(dgnsGtbaseparam.getSgt());
        if (dgnsGtbaseparam.getIslegal()==0)return;
        calDgnsGtstandard.cal(new ParamForCalDgnsGtstandard(dgnsGtstandardList,dgnsGtstandardSetting,sgtList) {
            @Override
            public void add() {
                DgnsGtstandard dgnsGtstandard = new DgnsGtstandard();
                dgnsGtstandard.setId(UUID.randomUUID().toString());
                dgnsGtstandard.setWellname(dgnsGtbaseparam.getWellname());//井号
                dgnsGtstandard.setWellId(dgnsGtbaseparam.getWellId());//id
                dgnsGtstandard.setSgtprodate(dgnsGtbaseparam.getProdDate());//生产时间
                dgnsGtstandard.setCreatetime(LocalDateTime.now());//创建时间
                dgnsGtstandard.setSgtcharacters(dgnsGtbaseparam.getSgt());//示功图
                dgnsGtstandard.setLatestupdatedtime(LocalDateTime.now());
                ThriftClientResponseData<Integer> thriftClientResponseData = gtComputerViewRecognitionClient.tryGTPredictType(sgtList, sgtList.size());
                DgnsGtmloutcode mlcode = dgnsGtmloutcodeService.getOne(new QueryWrapper<DgnsGtmloutcode>().eq("mlcode", thriftClientResponseData.getData()));
                if (mlcode!=null){
                    dgnsGtstandard.setGraphicfeatureinterpretation(mlcode.getGkdec());//特征解释，即 诊断的工况类型
                    dgnsGtstandard.setAppearcounter(1);//出现次数，默认第一次新增都是1；
                    addedDgnsGtstandardList.add(dgnsGtstandard); //保存标准功图数据
                    dgnsGtstandardList.add(dgnsGtstandard);
                }
            }

            @Override
            public void update(DgnsGtstandard dgnsGtstandard, Double similarityNum) {
                //更新 最大、最小相似度
                if(dgnsGtstandard.getMaxsemblance()==null || similarityNum > dgnsGtstandard.getMaxsemblance()){
                    dgnsGtstandard.setMaxsemblance(similarityNum);
                }
                if(dgnsGtstandard.getMinsemblance()==null || similarityNum < dgnsGtstandard.getMinsemblance()){
                    dgnsGtstandard.setMinsemblance(similarityNum);
                }
                //出现次数+1
                dgnsGtstandard.setAppearcounter(dgnsGtstandard.getAppearcounter()+1);
                dgnsGtstandard.setLatestupdatedtime(LocalDateTime.now());//最后更新日期
                //保存更新
                addedDgnsGtstandardList.add(dgnsGtstandard);
                //updateById(dgnsGtstandard);
            }

            @Override
            public Double getSimilarityNum(List<List<Double>> sgtList, DgnsGtstandard dgnsGtstandard) {
                int failedCount = 0;
                ThriftClientResponseData<Double> doubleThriftClientResponseData;
                do{
                    // if failed retry 10 times
                    if(failedCount>=1){
                        try{
                            Thread.sleep(100);
                        }catch (Exception e){
                            e.printStackTrace();
                        }
                    }
                    List<List<Double>> sgtcharactersList = SGTUtil.encodeToUseList(dgnsGtstandard.getSgtcharacters());
                    doubleThriftClientResponseData = gtComputerViewRecognitionClient.trySGTSimilarByGrayVal(sgtList, sgtList.size(), sgtcharactersList, sgtcharactersList.size());
                }while (!doubleThriftClientResponseData.isSuccess()&&failedCount++<10);
                if(failedCount==10){
                    log.error("thrift call failed");
                    throw new RuntimeException("[DGNS]thrift call failed");
                }else {
                    return doubleThriftClientResponseData.getData();
                }
            }

            @Override
            public void remove(DgnsGtstandard dgnsGtstandard) {
                removeById(dgnsGtstandard.getId());
            }
        });

    }


    @Override
    public List<DgnsGtstandard> getNearlyMostAppearCounter(String wellId, String date) {
        return baseMapper.getNearlyMostAppearCounter(wellId,date);
    }
}

