package com.gct.diagnose.service.impl.core;

import com.gct.common.util.SGTUtil;
import com.gct.diagnose.entity.DgnsGtcvrresult;
import com.gct.diagnose.entity.DgnsGtstandard;
import com.gct.diagnose.rocketmq.DiagnoseMsg;
import com.gct.diagnose.service.DgnsGtstandardService;
import com.gct.diagnose.thriftclient.GTComputerViewRecognitionClient;
import com.gct.diagnose.thriftclient.ThriftClientResponseData;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author xusirui 2021/6/29
 */
@Slf4j
@Component
public class FirstDgnsImpl implements FirstDgns {

    @Autowired
    GTComputerViewRecognitionClient gtComputerViewRecognitionClient;

    @Autowired
    DgnsGtstandardService dgnsGtstandardService;

    @Override
    public void accept(Visitor visitor) {
        visitor.cvResult.setResult1("-1"); //-1代表一次诊断不成功，默认为-1,如果成功会被修改为对应工况
        //转换数据
        DiagnoseMsg diagnoseMsg = visitor.diagnoseMsg;
        List<List<Double>> sgtList = SGTUtil.encodeToUseList(diagnoseMsg.getSgt());
        if (visitor.gtbaseparam.getIslegal()==0){//功图不合法 不进行一次诊断 ,todo: 是否需要标准功图？？？？
            visitor.appendLog("一次诊断：功图不合法,不执行一次诊断");
        }
        else {
            //调用一次诊断
            ThriftClientResponseData<Integer> integerThriftClientResponseData
                    = gtComputerViewRecognitionClient.tryGTPredictType(sgtList, sgtList.size());
            if (!integerThriftClientResponseData.isSuccess()) {
                visitor.appendLog("一次诊断：网络故障，无法执行一次诊断");
                log.error("Thrift调用失败");
            }else {
                visitor.cvResult.setResult1(integerThriftClientResponseData.getData().toString());
            }
        }

        //获取该井生产日期最近 7 天内出现次数最多的功图作为标准功图
        List<DgnsGtstandard> dgnsGtstandardList = dgnsGtstandardService.
                getNearlyMostAppearCounter(diagnoseMsg.getWellId(), diagnoseMsg.getProdDate());
        if (dgnsGtstandardList == null || dgnsGtstandardList.size() == 0) {
            visitor.appendLog("一次诊断：缺失标准功图");
            log.error("缺失标准功图 无法诊断");
        }else {
            Double similar = 0.0;
            DgnsGtstandard maxSimilarDgns = dgnsGtstandardList.get(0);
            for (DgnsGtstandard dgnsGtstandard : dgnsGtstandardList) {
                List<List<Double>> tmpList = SGTUtil.encodeToUseList(dgnsGtstandard.getSgtcharacters());
                ThriftClientResponseData<Double> doubleThriftClientResponseData =
                        gtComputerViewRecognitionClient.trySGTSimilarByGrayVal(tmpList, tmpList.size(), sgtList, sgtList.size());
                if (doubleThriftClientResponseData.isSuccess()){
                    if (doubleThriftClientResponseData.getData() > similar) {
                        similar = doubleThriftClientResponseData.getData();
                        maxSimilarDgns = dgnsGtstandard;
                    }
                }else {
                    visitor.appendLog("一次诊断：网络故障，无法执行相似度算法");
                }
            }
            visitor.standardGt = maxSimilarDgns;
            visitor.similar = similar;
        }
    }
}
