package com.gct.diagnose.service.impl.core;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.SGTUtil;
import com.gct.diagnose.decisiontree.DecisionTreeGenerator;
import com.gct.diagnose.decisiontree.DependentDataConstructor;
import com.gct.diagnose.decisiontree.DependentDataFieldEnum;
import com.gct.diagnose.entity.DgnsGtcardLib;
import com.gct.diagnose.entity.DgnsGtcvrresult;
import com.gct.diagnose.entity.DgnsGtmloutcode;
import com.gct.diagnose.entity.Organization;
import com.gct.diagnose.feignclient.BaseDataClient;
import com.gct.diagnose.rocketmq.DiagnoseMsg;
import com.gct.diagnose.service.DgnsGtcardLibService;
import com.gct.diagnose.service.DgnsGtcvrresultService;
import com.gct.diagnose.service.DgnsGtmloutcodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @author xusirui 2021/6/30
 */
@Component
public class GenerateResultImpl implements GenerateResult {

    @Autowired
    DgnsGtcardLibService dgnsGtcardLibService;

    @Autowired
    DgnsGtmloutcodeService dgnsGtmloutcodeService;

    @Autowired
    DgnsGtcvrresultService dgnsGtcvrresultService;

    @Autowired
    BaseDataClient baseDataClient;

    @Override
    public void accept(Visitor visitor) {
        DiagnoseMsg diagnoseMsg = visitor.diagnoseMsg;
        DgnsGtcvrresult cvResult = visitor.cvResult;
        cvResult.setId(UUID.randomUUID().toString())
                .setWellId(diagnoseMsg.getWellId())
                .setSgt(SGTUtil.encodeToUseList(diagnoseMsg.getSgt()).toString())
                .setProcessparam(generateProcessParam(visitor))
        .setProdDateTime(LocalDateTime.parse(diagnoseMsg.getProdDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        dgnsGtcvrresultService.asyncAddCvrresult(cvResult);
    }

    private String generateProcessParam(Visitor visitor) {
        Map<String, Object> processparam = new HashMap<>();
        Map<String, Object> initMap = visitor.initMap;
        DependentDataConstructor globeData = visitor.globeData;
        processparam.put("SJ", initMap.get("curGtTime"));
        //传入井名字给前端，JH代表 name，wellname代表 id
        processparam.put(DependentDataFieldEnum.JH.getEnDec(),
                visitor.diagnoseMsg.getWellName());
        processparam.put(DependentDataFieldEnum.BS.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BS.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZBS.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZBS.getEnDec()));
        processparam.put(DependentDataFieldEnum.BJ.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BJ.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZBJ.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZBJ.getEnDec()));
        processparam.put(DependentDataFieldEnum.CC.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.CC.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZCC.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZCC.getEnDec()));
        processparam.put(DependentDataFieldEnum.CCi.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.CCi.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZCCi.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZCCi.getEnDec()));
        processparam.put(DependentDataFieldEnum.ZDZH.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.ZDZH.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZZDZH.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZZDZH.getEnDec()));
        processparam.put(DependentDataFieldEnum.ZXZH.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.ZXZH.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZZXZH.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZZXZH.getEnDec()));
        processparam.put(DependentDataFieldEnum.RCYe.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.RCYe.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZRCYe.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZRCYe.getEnDec()));
        processparam.put(DependentDataFieldEnum.RCY.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.RCY.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZRCY.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZRCY.getEnDec()));
        processparam.put(DependentDataFieldEnum.HSL.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.HSL.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZHSL.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZHSL.getEnDec()));
        processparam.put(DependentDataFieldEnum.DYM.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.DYM.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZDYM.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZDYM.getEnDec()));
        processparam.put(DependentDataFieldEnum.CMD.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.CMD.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZCMD.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZCMD.getEnDec()));
        visitor.appendLog(globeData.getGlobeDataValue(DependentDataFieldEnum.LOG.getEnDec()).toString());
        processparam.put(DependentDataFieldEnum.LOG.getEnDec(),
                visitor.logBuilder);
        processparam.put(DependentDataFieldEnum.BZGT.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZGT.getEnDec()));
        processparam.put(DependentDataFieldEnum.BZSJ.getEnDec(),
                globeData.getGlobeDataValue(DependentDataFieldEnum.BZSJ.getEnDec()));
        List<DgnsGtcardLib> libList = dgnsGtcardLibService.list(new QueryWrapper<DgnsGtcardLib>().eq("code", Integer.parseInt(visitor.cvResult.getResult1())));
        DgnsGtcardLib tzGt = null;
        if (libList == null || libList.size() == 0) {
            processparam.put("TZGT", null);
            processparam.put("TZSJ", null);
            processparam.put("TZJH", null);
            visitor.appendLog("诊断结果生成：无特征功图");
        } else {
            tzGt = libList.get(0);
            processparam.put("TZGT", SGTUtil.encodeToUseList(tzGt.getIndicatorDiagram()));
            processparam.put("TZSJ", tzGt.getProDateStr());
            processparam.put("TZJH", tzGt.getJh());
        }

        DgnsGtmloutcode dgnsGtmloutcode = dgnsGtmloutcodeService.getOne(
                new QueryWrapper<DgnsGtmloutcode>()
                        .eq("mlcode", Integer.parseInt(visitor.cvResult.getResult1())));
        processparam.put("TZ",dgnsGtmloutcode==null?null: dgnsGtmloutcode.getGkdec());
        //站点 作业区 井区
        processparam.put("ZD", visitor.diagnoseMsg.getOrgId());
        processparam.put("JQ", getParentOrgId(processparam.get("ZD").toString()));
        processparam.put("ZYQ", getParentOrgId(processparam.get("JQ").toString()));
        //标准工况
        processparam.put("BZGK", visitor.standardGt.getGraphicfeatureinterpretation());
        return JSONObject.toJSONString(processparam);
    }

    private String getParentOrgId(String orgId){
        Organization org = JSONObject.parseObject(
                JSONObject.parseObject(baseDataClient.getOrgById(orgId), Result.class).getData().toString()
                ,Organization.class);
        return org.getParentId();
    }
}
