package com.gct.diagnose.service.impl.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.diagnose.decisiontree.DecisionTree;
import com.gct.diagnose.decisiontree.DecisionTreeGenerator;
import com.gct.diagnose.decisiontree.DependentDataFieldEnum;
import com.gct.diagnose.decisiontree.TemplateConfiguration;
import com.gct.diagnose.entity.DgnsGtcvrparam;
import com.gct.diagnose.entity.DgnsGtmloutcode;
import com.gct.diagnose.rocketmq.DiagnoseMsg;
import com.gct.diagnose.service.DgnsGtcvrparamService;
import com.gct.diagnose.service.DgnsGtmloutcodeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.naming.OperationNotSupportedException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author xusirui 2021/6/29
 */
@Slf4j
@Component
public class SecondDgnsImpl implements SecondDgns {

    @Autowired
    DgnsGtmloutcodeService dgnsGtmloutcodeService;

    @Autowired
    DgnsGtcvrparamService dgnsGtcvrparamService;

    @Override
    public void accept(Visitor visitor) {
        //数据初始化
        DiagnoseMsg diagnoseMsg = visitor.diagnoseMsg;
        DecisionTreeGenerator generator = TemplateConfiguration.getApplicationContext().getBean(DecisionTreeGenerator.class);
        Map<String, Object> initMap = new HashMap<>();
        initMap.put(DependentDataFieldEnum.JH.getEnDec(), diagnoseMsg.getWellId());
        initMap.put("orgId", diagnoseMsg.getOrgId());
        initMap.put("curGtTime", diagnoseMsg.getProdDate());
        initMap.put(DependentDataFieldEnum.BZSJ.getEnDec(), visitor.standardGt.getSgtprodate());
        initMap.put(DependentDataFieldEnum.XSD.getEnDec(), visitor.similar);
        initMap.put("dgnsGtbaseparam", visitor.gtbaseparam);
        visitor.initMap = initMap;
        try {
            generator.setArgs(initMap);
        } catch (NoSuchFieldException e) {
            visitor.appendLog("二次诊断：运行参数不存在");
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            visitor.appendLog("二次诊断：非法操作");
            e.printStackTrace();
        }
        visitor.globeData = generator.getGlobeData();
        if (visitor.cvResult.getResult1().equals("-1")) {
            visitor.appendLog("二次诊断：一次诊断未成功进行,不执行二次诊断");
        } else {  //仅当数据合法且一次诊断成功运行二次诊断
            DgnsGtmloutcode gtmloutcode = dgnsGtmloutcodeService.getOne(new QueryWrapper<DgnsGtmloutcode>().eq("mlcode", Integer.parseInt(visitor.cvResult.getResult1())));
            DgnsGtcvrparam gtcvrparam = dgnsGtcvrparamService.getOne(new QueryWrapper<DgnsGtcvrparam>().eq("paramid", gtmloutcode == null ? null : gtmloutcode.getGkcode()));
            if (gtmloutcode == null) {
                visitor.appendLog("二次诊断：找不到一次诊断对应工况");
                log.warn("二次诊断：找不到一次诊断对应工况："+visitor.cvResult.getResult1());
            } else if (gtcvrparam == null) {
                visitor.appendLog("二次诊断：找不到该工况的诊断算法");
                log.warn("二次诊断：找不到该工况的诊断算法："+gtmloutcode.getGkdec());
            } else {
                DecisionTree tree = generator.generateDecisionTreeFromJson(gtcvrparam.getParamcontent());
                try {
                    visitor.cvResult.setResult2(Objects.requireNonNull(generator.runDecisionTree(tree)).toString());
                } catch (OperationNotSupportedException | IllegalStateException e) {
                    visitor.appendLog("二次诊断：操作不支持或数据不存在");
                    log.warn("二次诊断：操作不支持或数据不存在");
                }
            }
            if (visitor.cvResult.getResult2() == null) visitor.cvResult.setResult2("二次诊断失败");
        }
    }
}
