package com.gct.diagnose.thriftclient;


import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * <p>
 * ThriftClient: for diagram_card recognition
 * <p>
 * use @Autowired to inject your dependence location
 * </p>
 *
 * @author lloyd
 * @version 1.0
 * @since 2021.1.21
 */
@Slf4j
@Component
public class GTComputerViewRecognitionClient {
    private static final int DEFAULTPREDICTTYPE = -1;
    private static final double DEFAUSIMILAR = -1;

    private final ThriftClientFactory factory;
    gtcvrService.Client client;
    ReadWriteLock lockClient = new ReentrantReadWriteLock();

    @Autowired
    public GTComputerViewRecognitionClient(ThriftClientFactory factory) {
        this.factory = factory;
        try {
            client = factory.getClient();
        }catch (Exception e){
            e.printStackTrace();
        }

    }

    private void refreshClient() {
        lockClient.writeLock().lock();
        client = factory.getClient();
        lockClient.writeLock().unlock();
    }

    private int GTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        int result = DEFAULTPREDICTTYPE;
        lockClient.readLock().lock();
        try {
            result = client.GTPredictType(sgt, sgtPointNum);
        } catch (Exception e) {
            log.warn("RPC:ERROR:GTPredictType");
            e.printStackTrace();
        } finally {
            lockClient.readLock().unlock();
        }
        return result;
    }

    private double SGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {
        double result = DEFAUSIMILAR;
        lockClient.readLock().lock();
        try {
            result = client.SGTSimillarByGrayVal(baseSGT, baseSGTPointNum, matchSGT, matchSGTPointNum);
        } catch (Exception e) {
            log.warn("RPC:ERROR:SGTSimilarByGrayVal");
        } finally {
            lockClient.readLock().unlock();
        }
        return result;
    }

    /**
     * 进行功图图形识别
     *
     * @param sgt         功图
     * @param sgtPointNum 功图点数
     * @return ResponseData
     */
    public synchronized ThriftClientResponseData<Integer> tryGTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        int pType = GTPredictType(sgt, sgtPointNum);
        if (pType == DEFAULTPREDICTTYPE) {
            return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);
        } else return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
        /*int failedCount = 0;
        do {
            int pType = GTPredictType(sgt, sgtPointNum);
            if (pType == DEFAULTPREDICTTYPE) {
                try {
                    Thread.sleep(50);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
            }
            refreshClient();
        } while (failedCount++ < 5);
        return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);*/
    }

    /**
     * 灰度计算两幅功图的特征相似度
     *
     * @param baseSGT          功图
     * @param baseSGTPointNum  功图点数
     * @param matchSGT         功图
     * @param matchSGTPointNum 功图点数
     * @return ResponseData
     */
    public synchronized ThriftClientResponseData<Double> trySGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {
        double pSimilar = SGTSimilarByGrayVal(baseSGT, baseSGTPointNum, matchSGT, matchSGTPointNum);
        if (pSimilar == DEFAULTPREDICTTYPE)
            return new ThriftClientResponseData<Double>().setData(null).setSuccess(false);
        else return new ThriftClientResponseData<Double>().setData(pSimilar).setSuccess(true);
    }
}
